# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class SideConfigurationDataRequest(BaseModel):
    """
    Configuration needed to define a side. Sides are referenced by Label. Beyond that, other properties  can be used to reference either transaction fields, or transaction properties.  # noqa: E501
    """
    side:  StrictStr = Field(...,alias="side", description="The side's label.") 
    security:  StrictStr = Field(...,alias="security", description="The security, or instrument.") 
    currency:  StrictStr = Field(...,alias="currency", description="The currency.") 
    rate:  StrictStr = Field(...,alias="rate", description="The rate.") 
    units:  StrictStr = Field(...,alias="units", description="The units.") 
    amount:  StrictStr = Field(...,alias="amount", description="The amount.") 
    __properties = ["side", "security", "currency", "rate", "units", "amount"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SideConfigurationDataRequest:
        """Create an instance of SideConfigurationDataRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SideConfigurationDataRequest:
        """Create an instance of SideConfigurationDataRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SideConfigurationDataRequest.parse_obj(obj)

        _obj = SideConfigurationDataRequest.parse_obj({
            "side": obj.get("side"),
            "security": obj.get("security"),
            "currency": obj.get("currency"),
            "rate": obj.get("rate"),
            "units": obj.get("units"),
            "amount": obj.get("amount")
        })
        return _obj

SideConfigurationDataRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.model_property import ModelProperty

class CreateReferencePortfolioRequest(BaseModel):
    """
    CreateReferencePortfolioRequest
    """
    display_name:  StrictStr = Field(...,alias="displayName", description="The name of the reference portfolio.") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="A long form text description of the portfolio.") 
    code:  StrictStr = Field(...,alias="code", description="Unique identifier for the portfolio.") 
    created: Optional[datetime] = Field(default=None, description="The original creation date, defaults to today if not specified when creating a portfolio.")
    properties: Optional[Dict[str, ModelProperty]] = Field(default=None, description="Portfolio properties to add to the portfolio.")
    instrument_scopes: Optional[List[StrictStr]] = Field(default=None, description="Instrument Scopes.", alias="instrumentScopes")
    base_currency:  Optional[StrictStr] = Field(None,alias="baseCurrency", description="The base currency of the transaction portfolio in ISO 4217 currency code format.") 
    __properties = ["displayName", "description", "code", "created", "properties", "instrumentScopes", "baseCurrency"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateReferencePortfolioRequest:
        """Create an instance of CreateReferencePortfolioRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if created (nullable) is None
        # and __fields_set__ contains the field
        if self.created is None and "created" in self.__fields_set__:
            _dict['created'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        # set to None if instrument_scopes (nullable) is None
        # and __fields_set__ contains the field
        if self.instrument_scopes is None and "instrument_scopes" in self.__fields_set__:
            _dict['instrumentScopes'] = None

        # set to None if base_currency (nullable) is None
        # and __fields_set__ contains the field
        if self.base_currency is None and "base_currency" in self.__fields_set__:
            _dict['baseCurrency'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateReferencePortfolioRequest:
        """Create an instance of CreateReferencePortfolioRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateReferencePortfolioRequest.parse_obj(obj)

        _obj = CreateReferencePortfolioRequest.parse_obj({
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "code": obj.get("code"),
            "created": obj.get("created"),
            "properties": dict(
                (_k, ModelProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "instrument_scopes": obj.get("instrumentScopes"),
            "base_currency": obj.get("baseCurrency")
        })
        return _obj

CreateReferencePortfolioRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.relational_dataset_field_definition import RelationalDatasetFieldDefinition
from lusid.models.resource_id import ResourceId

class CreateRelationalDatasetDefinitionRequest(BaseModel):
    """
    CreateRelationalDatasetDefinitionRequest
    """
    id: ResourceId
    display_name:  StrictStr = Field(...,alias="displayName", description="A user-friendly display name for the relational dataset definition.") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="A detailed description of the relational dataset definition and its purpose.") 
    applicable_entity_types: List[StrictStr] = Field(description="The types of entities this relational dataset definition can be applied to (e.g. Instrument, Portfolio, etc.).", alias="applicableEntityTypes")
    field_schema: List[RelationalDatasetFieldDefinition] = Field(description="The schema defining the structure and data types of the relational dataset.", alias="fieldSchema")
    __properties = ["id", "displayName", "description", "applicableEntityTypes", "fieldSchema"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateRelationalDatasetDefinitionRequest:
        """Create an instance of CreateRelationalDatasetDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of id
        if self.id:
            _dict['id'] = self.id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in field_schema (list)
        _items = []
        if self.field_schema:
            for _item in self.field_schema:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fieldSchema'] = _items
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateRelationalDatasetDefinitionRequest:
        """Create an instance of CreateRelationalDatasetDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateRelationalDatasetDefinitionRequest.parse_obj(obj)

        _obj = CreateRelationalDatasetDefinitionRequest.parse_obj({
            "id": ResourceId.from_dict(obj.get("id")) if obj.get("id") is not None else None,
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "applicable_entity_types": obj.get("applicableEntityTypes"),
            "field_schema": [RelationalDatasetFieldDefinition.from_dict(_item) for _item in obj.get("fieldSchema")] if obj.get("fieldSchema") is not None else None
        })
        return _obj

CreateRelationalDatasetDefinitionRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.mastered_instrument import MasteredInstrument

class CollateralInstrument(BaseModel):
    """
    Wrapper for one instrument in a larger collateral basket, as part of a repurchase agreement modelled as a FlexibleRepo.  # noqa: E501
    """
    units: Union[StrictFloat, StrictInt] = Field(description="The amount of the instrument in the basket for this repurchase agreement.")
    instrument: MasteredInstrument
    __properties = ["units", "instrument"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CollateralInstrument:
        """Create an instance of CollateralInstrument from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of instrument
        if self.instrument:
            _dict['instrument'] = self.instrument.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CollateralInstrument:
        """Create an instance of CollateralInstrument from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CollateralInstrument.parse_obj(obj)

        _obj = CollateralInstrument.parse_obj({
            "units": obj.get("units"),
            "instrument": MasteredInstrument.from_dict(obj.get("instrument")) if obj.get("instrument") is not None else None
        })
        return _obj

CollateralInstrument.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class SideDefinitionRequest(BaseModel):
    """
    SideDefinitionRequest
    """
    security:  StrictStr = Field(...,alias="security", description="The field or property key defining the side's security, or instrument.") 
    currency:  StrictStr = Field(...,alias="currency", description="The field or property key defining the side's currency.") 
    rate:  StrictStr = Field(...,alias="rate", description="The field or property key defining the side's rate.") 
    units:  StrictStr = Field(...,alias="units", description="The value, field or property key defining the side's units.") 
    amount:  StrictStr = Field(...,alias="amount", description="The value, field or property key defining the side's amount") 
    notional_amount:  Optional[StrictStr] = Field(None,alias="notionalAmount", description="The value, field or property key defining the side's notional amount") 
    current_face:  Optional[StrictStr] = Field(None,alias="currentFace", description="The value, field or property key defining the side's current face / outstanding notional.") 
    __properties = ["security", "currency", "rate", "units", "amount", "notionalAmount", "currentFace"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SideDefinitionRequest:
        """Create an instance of SideDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if notional_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.notional_amount is None and "notional_amount" in self.__fields_set__:
            _dict['notionalAmount'] = None

        # set to None if current_face (nullable) is None
        # and __fields_set__ contains the field
        if self.current_face is None and "current_face" in self.__fields_set__:
            _dict['currentFace'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SideDefinitionRequest:
        """Create an instance of SideDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SideDefinitionRequest.parse_obj(obj)

        _obj = SideDefinitionRequest.parse_obj({
            "security": obj.get("security"),
            "currency": obj.get("currency"),
            "rate": obj.get("rate"),
            "units": obj.get("units"),
            "amount": obj.get("amount"),
            "notional_amount": obj.get("notionalAmount"),
            "current_face": obj.get("currentFace")
        })
        return _obj

SideDefinitionRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.blocked_order_request import BlockedOrderRequest
from lusid.models.currency_and_amount import CurrencyAndAmount
from lusid.models.perpetual_property import PerpetualProperty
from lusid.models.resource_id import ResourceId

class BlockAndOrdersRequest(BaseModel):
    """
    BlockAndOrdersRequest
    """
    block_id: ResourceId = Field(alias="blockId")
    orders: List[BlockedOrderRequest] = Field(description="An order which belongs to a block. Fields common to both entities (such as instrument) should be derived from the block.")
    block_properties: Optional[Dict[str, PerpetualProperty]] = Field(default=None, description="Client-defined properties associated with this block.", alias="blockProperties")
    instrument_identifiers: Dict[str, Optional[StrictStr]] = Field(description="The instrument ordered.", alias="instrumentIdentifiers")
    side:  Optional[StrictStr] = Field(None,alias="side", description="The client's representation of the block's side (buy, sell, short, etc). BlockedOrders in the request which do not specify a side will have their side populated with this value.") 
    type:  Optional[StrictStr] = Field(None,alias="type", description="The block order's type (examples: Limit, Market, ...)") 
    time_in_force:  Optional[StrictStr] = Field(None,alias="timeInForce", description="The block orders' time in force (examples: Day, GoodTilCancel, ...)") 
    var_date: Optional[datetime] = Field(default=None, description="The date on which the block was made", alias="date")
    limit_price: Optional[CurrencyAndAmount] = Field(default=None, alias="limitPrice")
    stop_price: Optional[CurrencyAndAmount] = Field(default=None, alias="stopPrice")
    __properties = ["blockId", "orders", "blockProperties", "instrumentIdentifiers", "side", "type", "timeInForce", "date", "limitPrice", "stopPrice"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BlockAndOrdersRequest:
        """Create an instance of BlockAndOrdersRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of block_id
        if self.block_id:
            _dict['blockId'] = self.block_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in orders (list)
        _items = []
        if self.orders:
            for _item in self.orders:
                if _item:
                    _items.append(_item.to_dict())
            _dict['orders'] = _items
        # override the default output from pydantic by calling `to_dict()` of each value in block_properties (dict)
        _field_dict = {}
        if self.block_properties:
            for _key in self.block_properties:
                if self.block_properties[_key]:
                    _field_dict[_key] = self.block_properties[_key].to_dict()
            _dict['blockProperties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of limit_price
        if self.limit_price:
            _dict['limitPrice'] = self.limit_price.to_dict()
        # override the default output from pydantic by calling `to_dict()` of stop_price
        if self.stop_price:
            _dict['stopPrice'] = self.stop_price.to_dict()
        # set to None if block_properties (nullable) is None
        # and __fields_set__ contains the field
        if self.block_properties is None and "block_properties" in self.__fields_set__:
            _dict['blockProperties'] = None

        # set to None if side (nullable) is None
        # and __fields_set__ contains the field
        if self.side is None and "side" in self.__fields_set__:
            _dict['side'] = None

        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict['type'] = None

        # set to None if time_in_force (nullable) is None
        # and __fields_set__ contains the field
        if self.time_in_force is None and "time_in_force" in self.__fields_set__:
            _dict['timeInForce'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BlockAndOrdersRequest:
        """Create an instance of BlockAndOrdersRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BlockAndOrdersRequest.parse_obj(obj)

        _obj = BlockAndOrdersRequest.parse_obj({
            "block_id": ResourceId.from_dict(obj.get("blockId")) if obj.get("blockId") is not None else None,
            "orders": [BlockedOrderRequest.from_dict(_item) for _item in obj.get("orders")] if obj.get("orders") is not None else None,
            "block_properties": dict(
                (_k, PerpetualProperty.from_dict(_v))
                for _k, _v in obj.get("blockProperties").items()
            )
            if obj.get("blockProperties") is not None
            else None,
            "instrument_identifiers": obj.get("instrumentIdentifiers"),
            "side": obj.get("side"),
            "type": obj.get("type"),
            "time_in_force": obj.get("timeInForce"),
            "var_date": obj.get("date"),
            "limit_price": CurrencyAndAmount.from_dict(obj.get("limitPrice")) if obj.get("limitPrice") is not None else None,
            "stop_price": CurrencyAndAmount.from_dict(obj.get("stopPrice")) if obj.get("stopPrice") is not None else None
        })
        return _obj

BlockAndOrdersRequest.update_forward_refs()
